@@ -3,6 +3,7 @@
 #   recenter=TRUE -> result has same means as original, else means = 0 for cols 2:p
 #   rescale=TRUE -> result has same sd as original, else, sd = residual sd
 #   adjnames=TRUE -> colnames are adjusted to Y1, Y2.1, Y3.12, ...
+#  12-5-2010:  Fixed buglet when matrix has no row/col names
 
 gsorth <- function(y, order, recenter=TRUE, rescale=TRUE, adjnames=TRUE) {
 	n <- nrow(y)
@@ -22,8 +23,8 @@ gsorth <- function(y, order, recenter=TRUE, rescale=TRUE, adjnames=TRUE) {
 	zsd <- sd(z)
 	if (rescale) z <- z %*% diag( ysd/zsd )
 	if (recenter) z <- z + matrix(rep(ybar,times=n), ncol=p, byrow=TRUE)
-	rownames(z) <- rownames(y)
-	colnames(z) <- colnames(y)
+	rownames(z) <- rownames(y, do.NULL=FALSE)
+	colnames(z) <- colnames(y, do.NULL=FALSE)
 	if (adjnames) {
 		for (j in 2:p) {
 			colnames(z)[j] <- paste(colnames(z)[j], '.', sep="",
